# -*- coding: utf-8 -*-
import re

from django.conf import settings
from django.contrib.auth.models import AnonymousUser
from django.http import HttpRequest
from django.template import RequestContext
from django.template.loader import render_to_string
from django.utils.six.moves import html_parser

from haystack import indexes

from glitter.page import Glitter
from glitter.pages.models import Page


class PageCharField(indexes.CharField):
    def prepare_template(self, obj):
        template_name = 'search/indexes/%s/%s_%s.txt' % (
            obj._meta.app_label, obj._meta.model_name, self.instance_name)

        # Need to fake an HTTP request incase any templates aren't happy
        try:
            server_name = getattr(settings, 'ALLOWED_HOSTS', '127.0.0.1')[0]
        except IndexError:
            server_name = '127.0.0.1'

        request = HttpRequest()
        request.META = {
            'SERVER_NAME': server_name,
            'SERVER_PORT': 80,
        }
        request.path = request.path_info = obj.get_absolute_url()
        request.method = 'GET'
        request.user = AnonymousUser()

        # Create Glitter to render
        current_version = obj.current_version
        glitter = Glitter(page_version=current_version, request=request)

        # Render the template into a variable
        columns = glitter.render()
        content = render_to_string(template_name, {
            'glitter': glitter,
            'edit_mode': False,
            'columns': columns,
            obj._meta.model_name: obj,
            'object': obj,
        }, context_instance=RequestContext(request))

        # Need to escape HTML entities
        htmlparser = html_parser.HTMLParser()
        unescape = htmlparser.unescape
        content = unescape(content)

        # Prune excessive whitespace
        content = re.sub(r'\s+', ' ', content, flags=re.MULTILINE)

        return content


class PageIndex(indexes.SearchIndex, indexes.Indexable):
    text = PageCharField(document=True, use_template=True)

    def get_model(self):
        return Page

    def index_queryset(self, using=None):
        return self.get_model().objects.filter(
            published=True, current_version__isnull=False
        ).exclude(
            login_required=True
        )



# -*- coding: utf-8 -*-

from glitter.models import BaseBlock

from django.db import models
from django.utils.encoding import python_2_unicode_compatible


@python_2_unicode_compatible
class SampleModel(models.Model):
    content = models.TextField(blank=True)

    def __str__(self):
        return self.content


class SampleModelWithInlinesBlock(BaseBlock):
    pass


@python_2_unicode_compatible
class SampleInline(models.Model):
    parent_block = models.ForeignKey(SampleModelWithInlinesBlock)
    foreign_model = models.ForeignKey(SampleModel, on_delete=models.PROTECT)

    def __str__(self):
        return '%s' % (self.foreign_model,)



import django.dispatch


page_version_saved = django.dispatch.Signal(providing_args=['obj', 'version', 'user'])
page_version_published = django.dispatch.Signal(providing_args=[
    'obj', 'version', 'previous_version', 'user'
])
page_version_unpublished = django.dispatch.Signal(providing_args=['obj', 'version', 'user'])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('glitter', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='TextTextBlock',
            fields=[
                ('id', models.AutoField(primary_key=True, auto_created=True, serialize=False, verbose_name='ID')),
                ('block_class', models.CharField(verbose_name='Class', max_length=50)),
                ('left_column', models.TextField()),
                ('right_column', models.TextField()),
                ('content_block', models.ForeignKey(editable=False, null=True, to='glitter.ContentBlock')),
            ],
            options={
                'verbose_name': 'two column text',
            },
        ),
    ]



# -*- coding: utf-8 -*-

from django.db import models

from glitter.models import BaseBlock


class BaseRedactorBlock(BaseBlock):
    block_class = models.CharField('Class', max_length=50)
    content = models.TextField()

    class Meta:
        abstract = True


class Redactor(BaseRedactorBlock):
    class Meta:
        verbose_name = 'text'



# -*- coding: utf-8 -*-

from django import forms


class ContactForm(forms.Form):
    name = forms.CharField()
    email = forms.EmailField()
    message = forms.CharField(widget=forms.Textarea)

    error_css_class = 'error'
    required_css_class = 'required'



# -*- coding: utf-8 -*-

from functools import update_wrapper

from django.apps import apps
from django.contrib import admin
from django.contrib.admin.models import CHANGE, LogEntry
from django.contrib.admin.options import csrf_protect_m
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse, resolve
from django.db import transaction
from django.db.models import Q
from django.http import HttpResponseRedirect, JsonResponse
from django.shortcuts import get_object_or_404
from django.template.defaultfilters import slugify
from django.template.response import TemplateResponse
from django.utils.decorators import method_decorator
from django.utils.encoding import force_text
from django.views.decorators.http import require_POST

from .forms import get_addblock_form, get_movecolumn_form, get_newpagetemplateform, MoveBlockForm
from .models import ContentBlock, Version
from .page import Glitter
from .signals import page_version_published, page_version_saved, page_version_unpublished
from .templates import get_layout
from .utils import duplicate, JSONEncoderForHTML
from .views import render_page


require_post_m = method_decorator(require_POST)


class GlitterPagePublishedFilter(admin.SimpleListFilter):
    title = 'published'
    parameter_name = 'published'

    def lookups(self, request, model_admin):
        return (
            ('1', 'Yes'),
            ('0', 'No'),
        )

    def queryset(self, request, queryset):
        if self.value() == '1':
            return queryset.filter(published=True).exclude(current_version=None)

        if self.value() == '0':
            return queryset.filter(published=True, current_version__isnull=True)


class GlitterAdminMixin(object):
    list_filter = (GlitterPagePublishedFilter,)
    glitter_render = None

    def is_published(self, obj):
        return obj.published and obj.current_version_id is not None
    is_published.boolean = True
    is_published.short_description = 'Published'

    def get_urls(self):
        from django.conf.urls import url

        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(view)(*args, **kwargs)
            return update_wrapper(wrapper, view)

        info = self.model._meta.app_label, self.model._meta.model_name

        urlpatterns = [
            # Start a page with a brand new template
            url(r'^(?P<object_id>\d+)/template/$',
                wrap(self.page_template_view),
                name='%s_%s_template' % info),

            # Redirect a user to view the latest appropriate page
            url(r'^(?P<object_id>\d+)/redirect/$',
                wrap(self.page_redirect_view),
                name='%s_%s_redirect' % info),

            # View and edit
            url(r'^editor/view/(?P<version_id>\d+)/$',
                wrap(self.page_version_view),
                name='%s_%s_version' % info),
            url(r'^editor/edit/(?P<version_id>\d+)/$',
                wrap(self.page_edit_view),
                name='%s_%s_edit' % info),

            # Change template
            url(r'^editor/changetemplate/(?P<version_id>\d+)/$',
                wrap(self.page_changetemplate_view),
                name='%s_%s_changetemplate' % info),

            # Save, publish, discard
            url(r'^edit/page/(?P<version_id>\d+)/save/$',
                wrap(self.page_save_view),
                name='%s_%s_save' % info),
            url(r'^edit/page/(?P<version_id>\d+)/publish/$',
                wrap(self.page_publish_view),
                name='%s_%s_publish' % info),
            url(r'^edit/page/(?P<version_id>\d+)/unpublish/$',
                wrap(self.page_unpublish_view),
                name='%s_%s_unpublish' % info),
            url(r'^edit/page/(?P<version_id>\d+)/discard/$',
                wrap(self.page_discard_view),
                name='%s_%s_discard' % info),

            # Block add
            url(r'^edit/page/(?P<version_id>\d+)/block/add/$',
                wrap(self.page_block_add_view),
                name='%s_%s_block_add' % info),

            # Block editing/updating/moving
            url(r'^edit/block/del/(?P<contentblock_id>\d+)/$',
                wrap(self.page_block_delete_view),
                name='%s_%s_block_delete' % info),
            url(r'^edit/block/move/(?P<contentblock_id>\d+)/$',
                wrap(self.page_block_move_view),
                name='%s_%s_block_move' % info),
            url(r'^edit/block/column/(?P<contentblock_id>\d+)/$',
                wrap(self.page_block_column_view),
                name='%s_%s_block_column' % info),

        ] + super(GlitterAdminMixin, self).get_urls()
        return urlpatterns

    def has_edit_permission(self, request, obj=None, version=None):
        # Has the edit permission for this object type
        can_edit = request.user.has_perm('{}.edit_{}'.format(
            self.opts.app_label, self.opts.model_name
        ))

        if can_edit and version is not None:
            # Version must not be saved, and must belong to this user
            if version.version_number or version.owner != request.user:
                can_edit = False

        return can_edit

    def has_publish_permission(self, request, obj=None):
        return request.user.has_perm('{}.publish_{}'.format(
            self.opts.app_label, self.opts.model_name
        ))

    def response_add(self, request, obj, *args, **kwargs):
        if '_saveandedit' in request.POST:
            return self.page_redirect(request, obj)
        else:
            return super(GlitterAdminMixin, self).response_add(request, obj, *args, **kwargs)

    def response_change(self, request, obj, *args, **kwargs):
        if '_saveandedit' in request.POST:
            return self.page_redirect(request, obj)
        else:
            return super(GlitterAdminMixin, self).response_change(request, obj, *args, **kwargs)

    @csrf_protect_m
    @transaction.atomic
    def page_template_view(self, request, object_id):
        obj = get_object_or_404(self.model, id=object_id)

        if not self.has_edit_permission(request, obj):
            raise PermissionDenied

        NewPageTemplateForm = get_newpagetemplateform(model=self.model)
        form = NewPageTemplateForm(request.POST or None)

        if form.is_valid():
            version = form.save(commit=False)
            version.content_type = self.content_type
            version.object_id = obj.id
            version.owner = request.user
            version.save()

            opts = self.opts.app_label, self.opts.model_name
            return HttpResponseRedirect(reverse('admin:%s_%s_version' % opts, kwargs={
                'version_id': version.id,
            }))

        return TemplateResponse(request, 'admin/glitter/new_template.html', {
            'form': form,
        })

    def page_redirect(self, request, obj):
        opts = self.opts.app_label, self.opts.model_name

        if not self.has_edit_permission(request, obj):
            raise PermissionDenied

        # Redirect a user to the published page if it has one
        if obj.current_version:
            return HttpResponseRedirect(obj.get_absolute_url())

        # Otherwise we'll go for the latest version a user has access to
        try:
            version = Version.objects.filter(
                content_type=ContentType.objects.get_for_model(obj), object_id=obj.id).exclude(
                ~Q(owner=request.user), version_number__isnull=True)[0]
            return HttpResponseRedirect(reverse('admin:%s_%s_version' % opts, kwargs={
                'version_id': version.id,
            }))
        except IndexError:
            pass

        # Last resort is getting the user to create a new page with a new template
        return HttpResponseRedirect(reverse('admin:%s_%s_template' % opts, kwargs={
            'object_id': obj.id,
        }))

    def page_redirect_view(self, request, object_id):
        page = get_object_or_404(self.model, id=object_id)
        return self.page_redirect(request, page)

    def page_version_view(self, request, version_id):
        version = get_object_or_404(self.version_queryset().select_related(), id=version_id)

        obj = version.content_object

        if not self.has_edit_permission(request, obj):
            raise PermissionDenied

        # Deny other users from viewing an unsaved version
        if not version.version_number and version.owner != request.user:
            raise PermissionDenied

        if not self.glitter_render:
            func, args, kwargs = resolve(obj.get_absolute_url())
            kwargs['edit_mode'] = False
            kwargs['version'] = version
            response = func(request, *args, **kwargs)
            return response
        else:
            return render_page(request, obj, version)

    @property
    def content_type(self):
        return ContentType.objects.get_for_model(self.model)

    def version_queryset(self):
        return Version.objects.filter(content_type=self.content_type)

    def contentblock_queryset(self):
        return ContentBlock.objects.filter(obj_version__content_type=self.content_type)

    @csrf_protect_m
    @transaction.atomic
    def page_edit_view(self, request, version_id):
        opts = self.opts.app_label, self.opts.model_name

        version = get_object_or_404(self.version_queryset().select_related(), id=version_id)

        obj = version.content_object

        if not self.has_edit_permission(request, obj):
            raise PermissionDenied

        # Deny other users from viewing an unsaved version
        if not version.version_number and version.owner != request.user:
            raise PermissionDenied

        # POST request to initiate
        if request.method == 'POST':
            # No need to copy a version if they're still editing it
            if not version.version_number:
                return HttpResponseRedirect(reverse('admin:%s_%s_edit' % opts, kwargs={
                    'version_id': version_id,
                }))

            # Create a copy of this version for the user
            new_version = Version.objects.create(
                content_type=self.content_type,
                object_id=obj.id,
                template_name=version.template_name,
                owner=request.user)

            for content_block in version.contentblock_set.all():
                # Copy the block
                new_block = duplicate(content_block.content_object)
                new_block.save()

                # Copy the content block
                new_content_block = content_block
                new_content_block.id = None
                new_content_block.content_object = new_block
                new_content_block.obj_version = new_version
                new_content_block.save()

                # Point the block back to the ContentBlock
                new_block.content_block = new_content_block
                new_block.save()

            return HttpResponseRedirect(reverse('admin:%s_%s_edit' % opts, kwargs={
                'version_id': new_version.id,
            }))

        # Redirect to view if the version is already saved
        if version.version_number:
            return HttpResponseRedirect(reverse('admin:%s_%s_version' % opts, kwargs={
                'version_id': version_id,
            }))

        if not self.glitter_render:
            request = request
            func, args, kwargs = resolve(obj.get_absolute_url())
            kwargs['edit_mode'] = True
            kwargs['version'] = version
            response = func(request, *args, **kwargs)
            return response
        else:
            return render_page(request, obj, version, edit=True)

    @csrf_protect_m
    @transaction.atomic
    def page_changetemplate_view(self, request, version_id):
        version = get_object_or_404(self.version_queryset().select_related(), id=version_id)
        obj = version.content_object

        if not self.has_edit_permission(request, obj):
            raise PermissionDenied

        # Can't change a saved version
        if version.version_number:
            raise PermissionDenied

        old_template = get_layout(template_name=version.template_name)
        NewPageTemplateForm = get_newpagetemplateform(model=self.model)
        form = NewPageTemplateForm(request.POST, instance=version)

        # Deny other users from viewing an unsaved version
        if not version.version_number and version.owner != request.user:
            raise PermissionDenied

        if form.is_valid():
            version = form.save()
            new_template = get_layout(template_name=version.template_name)

            # If any columns don't exist in the new template, default them to the first column
            for i in old_template._meta.columns:
                if i not in new_template._meta.columns:
                    # Find the first column to put this content in - sadly this is currently the
                    # first column alphabetically for consistency. In future this will be the first
                    # column defined (the most important one).
                    new_column = sorted(new_template._meta.columns.keys())[0]

                    # Find the last block in the new first column
                    last_block = ContentBlock.objects.filter(
                        obj_version=version, column=new_column).last()

                    if last_block is not None:
                        next_position = last_block.position + 1
                    else:
                        next_position = 1

                    # Append to the first column
                    for content in ContentBlock.objects.filter(obj_version=version, column=i):
                        content.column = new_column
                        content.position = next_position
                        content.save()
                        next_position += 1

        opts = self.opts.app_label, self.opts.model_name
        return HttpResponseRedirect(reverse('admin:%s_%s_edit' % opts, kwargs={
            'version_id': version_id,
        }))

    @csrf_protect_m
    @require_post_m
    @transaction.atomic
    def page_save_view(self, request, version_id):
        version = get_object_or_404(self.version_queryset().select_related(), id=version_id)
        obj = version.content_object

        if not self.has_edit_permission(request, obj):
            raise PermissionDenied

        # Deny other users from saving a users version
        if version.owner != request.user:
            raise PermissionDenied

        # No need for errors, save anything and only update unsaved versions
        if not version.version_number:
            version.generate_version()
            version.save()

            page_version_saved.send(
                sender=obj.__class__,
                obj=obj,
                version=version,
                user=request.user)

        opts = self.opts.app_label, self.opts.model_name
        return HttpResponseRedirect(reverse('admin:%s_%s_version' % opts, kwargs={
            'version_id': version_id,
        }))

    @csrf_protect_m
    @require_post_m
    @transaction.atomic
    def page_publish_view(self, request, version_id):
        version = get_object_or_404(self.version_queryset().select_related(), id=version_id)
        obj = version.content_object

        if not self.has_publish_permission(request, obj):
            raise PermissionDenied

        # Deny other users from publishing an unsaved version
        if not version.version_number and version.owner != request.user:
            raise PermissionDenied

        # Repeated publish - ignore
        if obj.current_version == version:
            return HttpResponseRedirect(obj.get_absolute_url())

        # Save the page if it isn't already
        if not version.version_number:
            version.generate_version()
            version.save()

            page_version_saved.send(
                sender=obj.__class__,
                obj=obj,
                version=version,
                publish=True,
                user=request.user)

        # Publish!
        previous_version = obj.current_version
        obj.current_version = version
        obj.save()

        page_version_published.send(
            sender=obj.__class__,
            obj=obj,
            version=version,
            previous_version=previous_version,
            user=request.user)

        message = 'Published version %d' % (version.version_number,)
        LogEntry.objects.log_action(
            user_id=request.user.pk,
            content_type_id=self.content_type.pk,
            object_id=obj.id,
            object_repr=force_text(obj),
            action_flag=CHANGE,
            change_message=message
        )

        return HttpResponseRedirect(obj.get_absolute_url())

    @csrf_protect_m
    @require_post_m
    @transaction.atomic
    def page_unpublish_view(self, request, version_id):
        version = get_object_or_404(self.version_queryset().select_related(), id=version_id)
        obj = version.content_object

        if not self.has_publish_permission(request, obj):
            raise PermissionDenied

        # Not the current version? Just ignore the action and view the page again
        if obj.current_version == version:
            obj.current_version = None
            message = 'Unpublished page'

            page_version_unpublished.send(
                sender=obj.__class__,
                obj=obj,
                version=version,
                user=request.user)
            obj.save()

            LogEntry.objects.log_action(
                user_id=request.user.pk,
                content_type_id=self.content_type.pk,
                object_id=obj.id,
                object_repr=force_text(obj),
                action_flag=CHANGE,
                change_message=message
            )

        opts = self.opts.app_label, self.opts.model_name
        return HttpResponseRedirect(reverse('admin:%s_%s_version' % opts, kwargs={
            'version_id': version.id,
        }))

    @csrf_protect_m
    @transaction.atomic
    def page_discard_view(self, request, version_id):
        version = get_object_or_404(self.version_queryset().select_related(), id=version_id)
        obj = version.content_object

        if not self.has_edit_permission(request, obj, version=version):
            raise PermissionDenied

        # POST request to initiate
        if request.method == 'POST':
            # Remove all blocks
            for i in version.contentblock_set.all():
                i.delete()

            version.delete()

            return TemplateResponse(request, 'admin/glitter/version_discarded.html', {
                'obj': obj,
                'opts': self.model._meta,
            }, current_app=self.admin_site.name)

        return TemplateResponse(
            request, 'admin/glitter/version_discard.html', current_app=self.admin_site.name)

    @csrf_protect_m
    @transaction.atomic
    def page_block_add_view(self, request, version_id):
        version = get_object_or_404(self.version_queryset().select_related(), id=version_id)
        obj = version.content_object

        if not self.has_edit_permission(request, obj, version=version):
            raise PermissionDenied

        AddBlockForm = get_addblock_form(version)
        form = AddBlockForm(request.POST, instance=ContentBlock(obj_version=version))

        response_dict = {}

        if form.is_valid():
            # Figure out the block type we need
            new_obj_class = form.cleaned_data['block_type']
            app_name, model_name = new_obj_class.split('.', 1)
            new_obj_class = apps.get_model(app_name, model_name)

            # Create the block
            block = new_obj_class.objects.create()

            # Create a ContentBlock pointing to it
            content_block = form.save(commit=False)
            content_block.content_object = block

            if form.cleaned_data['top']:
                # User wants block at the top of the column, so set the position or it'll end up
                # being autosaved to the end of the column
                first_block = ContentBlock.objects.filter(
                    obj_version=version, column=content_block.column).first()

                if first_block is not None:
                    content_block.position = first_block.position - 1

            content_block.save()

            # Point the block back to the ContentBlock
            block.content_block = content_block
            block.save()

            # Updated the modified timestamp
            version.save()

            # Render the updated column
            response_dict['column'] = slugify(content_block.column)
            glitter = Glitter(version, request=request)
            columns = glitter.render(edit_mode=True, rerender=True)
            response_dict['content'] = columns[content_block.column]

        return JsonResponse(response_dict)

    @csrf_protect_m
    @transaction.atomic
    def page_block_delete_view(self, request, contentblock_id):
        content_block = get_object_or_404(self.contentblock_queryset(), id=contentblock_id)
        block = content_block.content_object
        version = content_block.obj_version
        obj = version.content_object

        if not self.has_edit_permission(request, obj, version=version):
            raise PermissionDenied

        if request.POST:
            # Save variables for use after deletion
            column = content_block.column

            # Delete the block
            content_block.delete()
            block.delete()

            # Render the updated column as a JSON object
            glitter = Glitter(version, request=request)
            columns = glitter.render(edit_mode=True, rerender=True)
            rendered_json = JSONEncoderForHTML().encode({
                'content': columns[column],
            })

            return TemplateResponse(request, 'admin/glitter/update_column.html', {
                'column': slugify(column),
                'rendered_json': rendered_json,
            }, current_app=self.admin_site.name)

        return TemplateResponse(request, 'admin/glitter/block_delete.html', {
            'content_block': block,
        }, current_app=self.admin_site.name)

    @csrf_protect_m
    @transaction.atomic
    def page_block_move_view(self, request, contentblock_id):
        content_block = get_object_or_404(self.contentblock_queryset(), id=contentblock_id)
        version = content_block.obj_version
        obj = version.content_object

        if not self.has_edit_permission(request, obj, version=version):
            raise PermissionDenied

        form = MoveBlockForm(request.POST)
        response_dict = {}

        if form.is_valid():
            move = form.cleaned_data['move']

            if move == MoveBlockForm.MOVE_UP or move == MoveBlockForm.MOVE_DOWN:
                # Move up/down involve swapping block positions

                try:
                    if move == MoveBlockForm.MOVE_UP:
                        other_block = ContentBlock.objects.filter(
                            obj_version=version,
                            column=content_block.column,
                            position__lt=content_block.position).order_by('-position')[0]
                    else:
                        other_block = ContentBlock.objects.filter(
                            obj_version=version,
                            column=content_block.column,
                            position__gt=content_block.position)[0]

                    old_position = content_block.position
                    new_position = other_block.position

                    # Temporarily unset other_block's position
                    other_block.position = None
                    other_block.save()

                    # Now set the appropriate positions
                    content_block.position = new_position
                    content_block.save()

                    other_block.position = old_position
                    other_block.save()

                except IndexError:
                    # User tried to move a block too far
                    pass

            else:
                if move == MoveBlockForm.MOVE_TOP:
                    # Move to top requires setting the position to one less than the first
                    other_block = ContentBlock.objects.filter(
                        obj_version=version, column=content_block.column)[0]

                    # This could be the first block
                    if content_block != other_block:
                        content_block.position = other_block.position - 1
                        content_block.save()
                else:
                    # Move bottom requires setting the position to one greater than the last
                    other_block = ContentBlock.objects.filter(
                        obj_version=version,
                        column=content_block.column).order_by('-position')[0]

                    # This could be the last block
                    if content_block != other_block:
                        content_block.position = other_block.position + 1
                        content_block.save()

            response_dict['column'] = slugify(content_block.column)
            glitter = Glitter(version, request=request)
            columns = glitter.render(edit_mode=True, rerender=True)
            response_dict['content'] = columns[content_block.column]

        return JsonResponse(response_dict)

    @csrf_protect_m
    @transaction.atomic
    def page_block_column_view(self, request, contentblock_id):
        content_block = get_object_or_404(self.contentblock_queryset(), id=contentblock_id)
        version = content_block.obj_version
        obj = version.content_object

        if not self.has_edit_permission(request, obj, version=version):
            raise PermissionDenied

        # Need to build a form which only has viable column moves
        template_obj = get_layout(template_name=version.template_name)
        column_choices = list(template_obj._meta.columns)
        column_choices.remove(content_block.column)

        MoveColumnForm = get_movecolumn_form(column_choices)
        form = MoveColumnForm(request.POST)
        response_dict = {}

        if form.is_valid():
            move = form.cleaned_data['move']
            source_column = content_block.column

            content_block.column = move

            # Find the last block of the column we're moving to
            if ContentBlock.objects.filter(
                    obj_version=version, column=move
            ).order_by('-position').exists():
                last_block = ContentBlock.objects.filter(
                    obj_version=version, column=move
                ).order_by('-position')[0]
                content_block.position = last_block.position + 1

            content_block.save()

            # Setup the page that can render both updated columns
            glitter = Glitter(version, request=request)
            columns = glitter.render(edit_mode=True, rerender=True)

            # Old column needs rendering again
            response_dict['source_column'] = slugify(source_column)
            response_dict['source_content'] = columns[source_column]

            # Now render the destination column
            response_dict['dest_column'] = slugify(content_block.column)
            response_dict['dest_content'] = columns[content_block.column]

        return JsonResponse(response_dict)



# -*- coding: utf-8 -*-

from django import forms
from django.conf import settings

from glitter import block_admin

from .models import TextTextBlock


class BaseTextTextBlockForm(forms.ModelForm):
    class Meta:
        model = TextTextBlock
        exclude = ()


def redactor_block_form():
    class_choices = getattr(settings, 'GLITTER_TEXTTEXT_CHOICES', ())

    # No classes? Just return a form without them
    if not class_choices:
        class TextTextBlockForm(BaseTextTextBlockForm):
            class Meta(BaseTextTextBlockForm.Meta):
                exclude = ('block_class',)

        return TextTextBlockForm

    # The more customisable version
    class_required = getattr(settings, 'GLITTER_TEXTTEXT_REQUIRED', False)

    class TextTextBlockForm(BaseTextTextBlockForm):
        block_class = forms.ChoiceField(
            label='Class', choices=class_choices, required=class_required)

    return TextTextBlockForm


class TextTextBlockAdmin(block_admin.BlockModelAdmin):
    form = redactor_block_form()


block_admin.site.register(TextTextBlock, TextTextBlockAdmin)
block_admin.site.register_block(TextTextBlock, 'Common')



# -*- coding: utf-8 -*-

import re

from django.core.exceptions import ValidationError


YOUTUBE_URL_RE = r"""
    (?x)^
    (
        (?:https?://|//)                                     # http(s):// or protocol-independent URL
        (?:(?:(?:(?:\w+\.)?[yY][oO][uU][tT][uU][bB][eE](?:-nocookie)?\.com/|
           youtube\.googleapis\.com/)                        # the various hostnames, with wildcard subdomains
        (?:.*?\#/)?                                          # handle anchor (#/) redirect urls
        (?:                                                  # the various things that can precede the ID:
            (?:(?:v|embed|e)/(?!videoseries))                # v/ or embed/ or e/
            |(?:                                             # or the v= param in all its forms
                (?:(?:watch|movie)(?:_popup)?(?:\.php)?/?)?  # preceding watch(_popup|.php) or nothing (like /?v=xxxx)
                (?:\?|\#!?)                                  # the params delimiter ? or # or #!
                (?:.*?&)??                                   # any other preceding param (like /?s=tuff&v=xxxx)
                v=
            )
        ))
        |(?:www\.)?cleanvideosearch\.com/media/action/yt/watch\?videoId=
        )
    )?                                                       # all until now is optional -> you can pass the naked ID
    ([0-9A-Za-z_-]{11})                                      # here is it! the YouTube video ID
    (?!.*?&list=)                                            # combined list/video URLs are handled by the playlist IE
    (?(1).+)?                                                # if we found the ID, everything can follow
    $
"""  # noqa

VIMEO_URL_RE = r"""
    (?x)
    https?://
    (?:(?:www|(?P<player>player))\.)?
    vimeo(?P<pro>pro)?\.com/
    (?!channels/[^/?#]+/?(?:$|[?#])|album/)
    (?:.*?/)?
    (?:(?:play_redirect_hls|moogaloop\.swf)\?clip_id=)?
    (?:videos?/)?
    (?P<id>[0-9]+)
    /?(?:[?&].*)?(?:[#].*)?$
"""


def validate_url(value):
    """ Validate url. """
    if not re.match(VIMEO_URL_RE, value) and not re.match(YOUTUBE_URL_RE, value):
        raise ValidationError('Invalid URL - only Youtube, Vimeo can be used.')



from django import template
from django.contrib.admin.templatetags.admin_modify import submit_row

register = template.Library()


@register.inclusion_tag('admin/glitter/page/submit_line.html', takes_context=True)
def glitter_submit_row(context):
    return submit_row(context)



# -*- coding: utf-8 -*-

from django import forms
from django.conf import settings

from .choices import POSITION_CHOICES
from .models import BaseTextImageBlock


class BaseTextImageForm(forms.ModelForm):
    class Meta:
        model = BaseTextImageBlock
        fields = ('position', 'image', 'content')

    position = forms.ChoiceField(
        choices=getattr(settings, 'GLITTER_TEXTIMAGE_CHOICES', POSITION_CHOICES))



# -*- coding: utf-8 -*-

from django.contrib.admin.sites import AdminSite
from django.contrib.admin.templatetags.admin_static import static
from django.contrib.admin.views.main import IS_POPUP_VAR, TO_FIELD_VAR
from django.contrib.admin.widgets import RelatedFieldWidgetWrapper
from django.core.urlresolvers import reverse
from django.forms.widgets import Media, Select
from django.template.loader import render_to_string

from glitter.assets.models import ImageCategory


class ImageRelatedFieldWidgetWrapper(RelatedFieldWidgetWrapper):
    def get_related_url(self, info, action, *args):
        url_params = '&'.join("%s=%s" % param for param in [
            (TO_FIELD_VAR, self.rel.get_related_field().name),
            (IS_POPUP_VAR, 1),
        ])

        return '{}?{}'.format(
            reverse(
                "admin:%s_%s_%s" % (info + (action,)), current_app=AdminSite(), args=args
            ), url_params
        )

    def render(self, name, value, *args, **kwargs):
        rel_to = self.rel.to
        info = (rel_to._meta.app_label, rel_to._meta.object_name.lower())
        self.widget.choices = self.choices
        context = self.widget.render(name, value, *args, **kwargs)
        context['name'] = name

        if self.can_add_related:
            related_url = self.get_related_url(info, 'add')
            context['can_add_related_url'] = related_url

        if self.can_change_related:
            related_url = self.get_related_url(info, 'change', '__fk__')
            context['can_change_related_url'] = related_url

        rendered = render_to_string(
            'admin/glitter/widgets/image_related_field.html', context
        )
        return rendered

    @property
    def media(self):
        js_media = Media(
            js=[
                static('admin/js/related-widget-wrapper.js'),
            ],
        )
        css_media = Media(
            css={
                'all': (static('glitter/css/widgets/images.css'),)
            }
        )
        return self.widget.media + js_media + css_media


class ImageSelect(Select):
    class Media:
        css = {
            'all': ('glitter/css/widgets/images.css',)
        }
        js = ('glitter/js/widgets/images.js',)

    def render(self, name, value, attrs=None, choices=()):
        if value is None:
            value = ''
        options = self.render_options(choices, [value])

        context = {
            'options': options,
            'images': self.choices.queryset.order_by('-created_at', 'modified_at', 'title',),
            'categories': ImageCategory.objects.all()
        }
        return context



# -*- coding: utf-8 -*-

from django.contrib.admin.sites import AdminSite

from glitter import block_admin
from glitter.assets.fields import AssetForeignKey
from glitter.assets.widgets import ImageRelatedFieldWidgetWrapper, ImageSelect

from .forms import BaseTextImageForm
from .models import TextImageBlock


class TextImageBlockAdmin(block_admin.BlockModelAdmin):
    form = BaseTextImageForm

    formfield_overrides = {
        AssetForeignKey: {
            'widget': ImageRelatedFieldWidgetWrapper(
                ImageSelect(),
                TextImageBlock._meta.get_field('image').rel,
                AdminSite(),
                can_add_related=True
            )
        }
    }


block_admin.site.register(TextImageBlock, TextImageBlockAdmin)
block_admin.site.register_block(TextImageBlock, 'Common')



#!/usr/bin/env python
from setuptools import find_packages, setup


install_requires = [
    'Django>=1.8,<1.9',
    'django-mptt>=0.7,<0.9',
    'django-mptt-admin>=0.3,<0.4',
    'sorl-thumbnail>=12.2',
]


setup(
    name='django-glitter',
    version='0.1.8',
    description='Glitter for Django',
    long_description=open('README.rst').read(),
    url='https://github.com/blancltd/django-glitter',
    maintainer='Blanc Ltd',
    maintainer_email='studio@blanc.ltd.uk',
    platforms=['any'],
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    license='BSD',
    install_requires=install_requires,
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('glitter', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SampleInline',
            fields=[
                ('id', models.AutoField(primary_key=True, verbose_name='ID', auto_created=True, serialize=False)),
            ],
        ),
        migrations.CreateModel(
            name='SampleModel',
            fields=[
                ('id', models.AutoField(primary_key=True, verbose_name='ID', auto_created=True, serialize=False)),
                ('content', models.TextField(blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='SampleModelWithInlinesBlock',
            fields=[
                ('id', models.AutoField(primary_key=True, verbose_name='ID', auto_created=True, serialize=False)),
                ('content_block', models.ForeignKey(null=True, editable=False, to='glitter.ContentBlock')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='sampleinline',
            name='foreign_model',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='sampleblocks.SampleModel'),
        ),
        migrations.AddField(
            model_name='sampleinline',
            name='parent_block',
            field=models.ForeignKey(to='sampleblocks.SampleModelWithInlinesBlock'),
        ),
    ]



from django.core.exceptions import ValidationError


def validate_page_url(value):
    if not value.startswith('/'):
        raise ValidationError('URL must start with a /')

    if not value.endswith('/'):
        raise ValidationError('URL must end with a /')

    if value.startswith('//'):
        raise ValidationError('Protocol relative URLs are not allowed')



import re

from django.contrib.syndication.views import add_domain
from django.template import RequestContext
from django.template.loader import render_to_string


def related_pages_generator(request, related_pages):
    for related_page in related_pages:
        if related_page.page:
            page_url = related_page.page.get_absolute_url()
            url = add_domain(request.get_host(), page_url, secure=request.is_secure())
        else:
            url = related_page.link

        # Remove common prefix
        url = re.sub(r'^https?://(?:www\.)?', '', url, flags=re.IGNORECASE)

        # Remove common suffix
        if url.endswith('/'):
            url = url.rstrip('/')

        yield related_page, url


def relatedpages_view(block, request, rerender, content_block, block_classes):
    css_classes = ' '.join(block_classes)

    related_pages = block.relatedpage_set.select_related('page').all()
    related_pages_links = related_pages_generator(request, related_pages)

    return render_to_string((
        'glitter/blocks/%s.html' % (block._meta.model_name,),
    ), {
        'content_block': content_block,
        'css_classes': css_classes,
        'object': block,
        'related_pages': related_pages,
        'related_pages_links': related_pages_links,
    }, context_instance=RequestContext(request))



# -*- coding: utf-8 -*-

from django.db import models


class GlitterManager(models.Manager):
    def published(self):
        return self.filter(published=True).exclude(current_version=None)

    def unpublished(self):
        return self.filter(published=True, current_version__isnull=True)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('glitter', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='HTML',
            fields=[
                ('id', models.AutoField(primary_key=True, verbose_name='ID', auto_created=True, serialize=False)),
                ('content', models.TextField()),
                ('content_block', models.ForeignKey(null=True, editable=False, to='glitter.ContentBlock')),
            ],
            options={
                'verbose_name': 'HTML',
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import mptt.fields
import glitter.pages.validators


class Migration(migrations.Migration):

    dependencies = [
        ('glitter', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Page',
            fields=[
                ('id', models.AutoField(verbose_name='ID', primary_key=True, serialize=False, auto_created=True)),
                ('published', models.BooleanField(default=True, db_index=True)),
                ('url', models.CharField(verbose_name='URL', validators=[glitter.pages.validators.validate_page_url], max_length=100, unique=True)),
                ('title', models.CharField(max_length=100)),
                ('login_required', models.BooleanField(default=False)),
                ('show_in_navigation', models.BooleanField(default=True, db_index=True)),
                ('unpublished_count', models.PositiveIntegerField(default=0, editable=False)),
                ('lft', models.PositiveIntegerField(db_index=True, editable=False)),
                ('rght', models.PositiveIntegerField(db_index=True, editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(db_index=True, editable=False)),
                ('current_version', models.ForeignKey(blank=True, editable=False, to='glitter.Version', null=True)),
                ('parent', mptt.fields.TreeForeignKey(blank=True, to='glitter_pages.Page', null=True, related_name='children')),
            ],
            options={
                'verbose_name': 'page',
                'ordering': ('url',),
                'default_permissions': ('add', 'change', 'delete', 'edit', 'publish'),
                'abstract': False,
                'permissions': (('view_protected_page', 'Can view protected page'),),
            },
        ),
    ]



# -*- coding: utf-8 -*-

from django.db import models
from django.utils.encoding import python_2_unicode_compatible


@python_2_unicode_compatible
class FileMixin(models.Model):
    class Meta:
        abstract = True
        ordering = ('-created_at', '-modified_at', 'title')

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return self.file.url

    def save(self, *args, **kwargs):
        # Avoid doing file size requests constantly
        self.file_size = self.file.size

        super(FileMixin, self).save(*args, **kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible

from glitter.assets.fields import AssetForeignKey
from glitter.fields import LinkField
from glitter.models import BaseBlock


@python_2_unicode_compatible
class BaseCarousel(models.Model):
    title = models.CharField(max_length=100, db_index=True)

    class Meta:
        abstract = True
        ordering = ('title',)

    def __str__(self):
        return self.title


class Carousel(BaseCarousel):
    pass


@python_2_unicode_compatible
class BaseCarouselImage(models.Model):
    carousel = models.ForeignKey(Carousel, related_name='carousel_images')
    title = models.CharField(max_length=100)
    subtitle = models.CharField(max_length=100, blank=True)
    image = AssetForeignKey('glitter_assets.Image', on_delete=models.PROTECT)
    link = LinkField()

    class Meta:
        ordering = ('id',)
        abstract = True

    def __str__(self):
        return self.title


class CarouselImage(BaseCarouselImage):
    pass


class CarouselBlock(BaseBlock):
    carousel = models.ForeignKey(Carousel, null=True, on_delete=models.PROTECT)

    class Meta:
        verbose_name = 'carousel'


class ImageOnlyCarousel(BaseCarousel):
    pass


@python_2_unicode_compatible
class BaseImageOnlyCarouselImage(models.Model):
    carousel = models.ForeignKey(ImageOnlyCarousel, related_name='carousel_images')
    image = AssetForeignKey('glitter_assets.Image', on_delete=models.PROTECT)

    class Meta:
        ordering = ('id',)
        abstract = True

    def __str__(self):
        if self.carousel:
            return '{}'.format(self.carousel)
        else:
            return 'Carousel image'


class ImageOnlyCarouselImage(BaseImageOnlyCarouselImage):
    pass


class ImageOnlyCarouselBlock(BaseBlock):
    carousel = models.ForeignKey(ImageOnlyCarousel, null=True, on_delete=models.PROTECT)

    class Meta:
        verbose_name = 'image only carousel'



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.db import models

from glitter.assets.fields import AssetForeignKey
from glitter.models import BaseBlock


class BaseTextImageBlock(BaseBlock):
    position = models.CharField(max_length=50)
    image = AssetForeignKey('glitter_assets.Image', null=True, on_delete=models.PROTECT)
    content = models.TextField()

    class Meta:
        abstract = True


class TextImageBlock(BaseTextImageBlock):
    class Meta:
        verbose_name = 'Text/Image'



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible

from .mixins import FileMixin


@python_2_unicode_compatible
class BaseCategory(models.Model):
    title = models.CharField(max_length=100, unique=True)

    class Meta:
        abstract = True
        ordering = ('title',)

    def __str__(self):
        return self.title


class FileCategory(BaseCategory):
    class Meta(BaseCategory.Meta):
        verbose_name_plural = 'file categories'


class File(FileMixin, models.Model):
    category = models.ForeignKey(FileCategory)
    title = models.CharField(max_length=100, db_index=True)
    file = models.FileField(upload_to='assets/file')
    file_size = models.PositiveIntegerField(default=0, editable=False)
    created_at = models.DateTimeField(auto_now_add=True)
    modified_at = models.DateTimeField(auto_now=True)


class ImageCategory(BaseCategory):
    class Meta(BaseCategory.Meta):
        verbose_name_plural = 'image categories'


class Image(FileMixin, models.Model):
    category = models.ForeignKey(ImageCategory)
    title = models.CharField(max_length=100, db_index=True)
    file = models.ImageField(
        'Image', upload_to='assets/image', height_field='image_height', width_field='image_width'
    )
    image_height = models.PositiveIntegerField(editable=False)
    image_width = models.PositiveIntegerField(editable=False)
    file_size = models.PositiveIntegerField(default=0, editable=False)
    created_at = models.DateTimeField(auto_now_add=True)
    modified_at = models.DateTimeField(auto_now=True)



# -*- coding: utf-8 -*-

from django.db.models.signals import post_save

from .models import Page


def version_update(instance, raw=False, **kwargs):
    # Don't update on loaddata
    if raw:
        return

    obj = instance.content_object

    if isinstance(obj, Page):
        # Page save method recalculates the unpublished pages
        obj.save()


post_save.connect(version_update, sender='glitter.Version')



# -*- coding: utf-8 -*-

from django.db import models

from glitter.assets.fields import AssetForeignKey
from glitter.fields import LinkField
from glitter.models import BaseBlock

from .managers import BaseImageBlockManager


class BaseImageBlock(BaseBlock):
    image = AssetForeignKey('glitter_assets.Image', null=True, on_delete=models.PROTECT)
    description = models.CharField(max_length=200, blank=True, help_text='Used as ALT text')
    caption = models.CharField(max_length=200, blank=True, help_text='Shown below the image')
    link = LinkField(blank=True)
    new_window = models.BooleanField('Open link in new window', default=False)

    objects = BaseImageBlockManager()

    class Meta:
        abstract = True


class ImageBlock(BaseImageBlock):
    class Meta:
        verbose_name = 'image'
